var fs = require("fs"),
    _ = require("lodash"),
    util = require("util");

module.exports = Settings;

function Settings(_path, options) {
    require("mkdirp")(require("path").dirname(_path));
    require("touch").sync(_path);
    var opts = _.extend({}, {skipReload: false, reloadSync: true, cb: function() {}}, options);
    this.path = _path;
    if (opts != null) {
        if (opts.skipReload !== true) if (opts.reloadSync) this.reloadSync(); else this.reload(opts.cb);
    }
}

Settings.prototype.reload = function(cb) {
    cb = cb || function() {};
    var self = this;
    fs.readFile(this.path, function(err, data) {
        if (!err) {
            try {
                self.raw_data = JSON.parse(data);
                cb(null);
            } catch (e) {
                cb(e);
            }
        } else {
            cb(err);
        }
    });
};
Settings.prototype.reloadSync = function() {
    try {
        this.raw_data = JSON.parse(fs.readFileSync(this.path));
    } catch (e) {
        this.raw_data = {};
        this.saveSync();
    }
};
Settings.prototype.save = function(cb) {
    cb = cb || function() {};
    fs.writeFile(this.path, JSON.stringify(this.raw_data), cb);
};
Settings.prototype.saveSync = function() {
    fs.writeFileSync(this.path, JSON.stringify(this.raw_data));
};
Settings.prototype.get = function(name, defaultValue) {
    var ret = this.raw_data[name];
    if (util.isNullOrUndefined(ret)) {
        this.set(name, defaultValue);
        return defaultValue;
    } else return ret;
};
Settings.prototype.set = function(name, value, options) {
    var opts = _.extend({}, {saveSync: true, save: false, cb: function() {}}, options);
    this.raw_data[name] = value;

    if (opts.saveSync === true) this.saveSync(); else if (opts.save === true) this.save(opts.cb);
};
Settings.prototype.setAll = function(toSet, options) {
    if (!util.isObject(toSet)) throw new Error("Expecting an object of keys/values to set ({option: \"value\"})");
    var opts = _.extend({}, {saveSync: true, save: false, cb: function() {}}, options);

    _.extend(this.raw_data, toSet);

    if (opts.saveSync === true) this.saveSync(); else if (opts.save === true) this.save(opts.cb);
};
Settings.prototype.getAll = function(names) {
    if (!util.isArray(names)) return this.get(names);

    var toReturn = {};
    for (var i = 0; i < names.length; i++) {
        if (util.isString(names[i])) {
            toReturn[names[i]] = this.raw_data[names[i]];
        } else if (util.isObject(names[i]) && util.isString(names[i].key)) {
            toReturn[names[i].key] = this.get(names[i].key, names[i].defaultValue);
        }
    }

    return toReturn;
};