var assert = require("chai").assert;
var Settings = require("../Settings");

describe("Settings", function() { //arrow functions discouraged in mocha docs.
    afterEach(function() {
        try {
            require("fs").unlinkSync("./temp.json");
        } catch (e) {}
    });
    describe("new()", function() {
        it("Should create a new Settings object", function() {
            assert.notEqual(new Settings("./temp.json"), null); //technically the constructor calls reloadSync by default so testing reloadSync shouldn't be needed but we'll include it below for completeness. Might remove later.
        });
    });
    describe("#reload()", function() {
        it("Should reload asynchronously and callback with no errors", function(done) {
            var settings = new Settings("./temp.json");
            settings.reload(function(err) {
                if (err) throw err;
                done();
            });
        });
    });
    describe("#reloadSync()", function() {
        it("Should reload synchronously", function() {
            var settings = new Settings("./temp.json");
            settings.reloadSync(); //doesn't actually ever throw an exception, not sure if test is needed. Included for completeness. Might remove later.
        });
    });
    describe("#save()", function() {
        it("Should save asynchronously and callback with no errors", function(done) {
            var settings = new Settings("./temp.json");
            settings.save(function(err) {
                if (err) throw err;
                done();
            });
        });
    });
    describe("#saveSync()", function() {
        it("Should save synchronously", function() {
            var settings = new Settings("./temp.json");
            settings.saveSync();
        });
    });
    describe("#set()", function() {
        it("Should set `one` to `temp`", function() {
            var settings = new Settings("./temp.json");
            settings.set("one", "temp");
            assert.equal(settings.get("one"), "temp");
        });
    });
    describe("#get()", function() {
        it("Should get the value `temp` from the key `one`", function() {
            var settings = new Settings("./temp.json");
            settings.set("one", "temp");
            assert.equal(settings.get("one"), "temp");
        });
        it("Should get the defaultValue for the nonexistent key `trg`", function() {
            var settings = new Settings("./temp.json");
            assert.equal(settings.get("trg", 3), 3);
        });
        it("Should set the defaultValue for the nonexistent key `trg1` and pull the correct value after set", function() {
            var settings = new Settings("./temp.json");
            var trg1 = settings.get("trg1", 3);
            assert.equal(settings.get("trg1", 5), 3);
        });
    });
    describe("#setAll()", function() {
        it("Should set the following key/values: 'temp' -> 'one', 'temp2' -> 'two'", function() {
            var settings = new Settings("./temp.json");
            settings.setAll({"temp": "one", "temp2": "two"});
            assert.equal((settings.get("temp") === "one") && (settings.get("temp2") === "two"), true);
        });
    });
    describe("#getAll()", function() {
        it("Should get all the correct values from the keys 'temp' and 'temp2'", function() {
            var settings = new Settings("./temp.json");
            var toSet = {"temp": "one", "temp2": "two"};
            settings.setAll(toSet);
            var obj = settings.getAll(["temp", "temp2"]);
            assert.equal(require("util").isObject(obj) && obj["temp"] == toSet["temp"] && obj["temp2"] == toSet["temp2"], true);
        });
        it("Should get all the correct defaultValue stuff from the keys 'temp3' and 'temp4", function() {
            var settings = new Settings("./temp.json");
            var got = settings.getAll([{key: "temp3", defaultValue: 3}, {key: "temp3", defaultValue: 9}, {key: "temp4",defaultValue: "one two three"}, {key: "temp4", defaultValue: "three two one"}]);
            assert.equal(require("util").isObject(got) && got["temp3"] === 3 && got["temp4"] === "one two three", true);
        });
    });
});